
import { csv } from 'd3';
import { performPCA } from './pca.js';

async function loadCSVData() {
  try {
    const data = await csv('https://raw.githubusercontent.com/ZeynebMhamedi/VizHub/main/full_gea.csv');
    const parsedData = data.map((row) => {
      const newRow = {};
      Object.keys(row).forEach((key) => {
        newRow[key] = isNaN(+row[key]) ? row[key] : +row[key];
      });
      return newRow;
    });

    const numericColumns = getNumericColumns(parsedData);
    const pcaResult = performPCA(numericColumns, 2);
    console.log('PCA Result:', pcaResult);
  } catch (error) {
    console.error('Error loading CSV file:', error);
  }
}

function getNumericColumns(data) {
  if (data.length === 0) return [];

  const numericData = [];
  const keys = Object.keys(data[0]).filter((key) => typeof data[0][key] === 'number');

  for (const row of data) {
    const numericRow = [];
    keys.forEach((key) => {
      numericRow.push(row[key]);
    });
    numericData.push(numericRow);
  }

  return numericData;
}

loadCSVData();
