// Simple implementation of PCA in JavaScript

function mean(matrix) {
  return matrix.map((row) => row.reduce((a, b) => a + b) / row.length);
}

function transpose(matrix) {
  return matrix[0].map((_, i) => matrix.map((row) => row[i]));
}

function dot(matrix1, matrix2) {
  const result = [];
  for (let i = 0; i < matrix1.length; i++) {
    result.push([]);
    for (let j = 0; j < matrix2[0].length; j++) {
      let sum = 0;
      for (let k = 0; k < matrix1[0].length; k++) {
        sum += matrix1[i][k] * matrix2[k][j];
      }
      result[i].push(sum);
    }
  }
  return result;
}

function covarianceMatrix(matrix) {
  const meanVector = mean(matrix);
  const meanCenteredMatrix = matrix.map((row) =>
    row.map((value, i) => value - meanVector[i])
  );
  const covarianceMatrix = dot(transpose(meanCenteredMatrix), meanCenteredMatrix);
  return covarianceMatrix.map((row, i) => row.map((value) => value / (matrix.length - 1)));
}

function getEigenVectors(matrix) {
  const eigResult = numeric.eig(matrix);
  return eigResult.E.x;
}

function projectData(data, eigenvectors, dimensions) {
  const projectedData = dot(data, eigenvectors.slice(0, dimensions));
  return projectedData;
}

function performPCA(data, dimensions) {
  const covMatrix = covarianceMatrix(data);
  const eigenvectors = getEigenVectors(covMatrix);
  return projectData(data, eigenvectors, dimensions);
}

export {
  performPCA
};
